/** KazgorColumn.java
 * Created on May 15, 2004
 *
 * Free.  Use however you see fit.
 */
package kazgorColumn;

import java.io.File;
import java.lang.String;




import org.gudy.azureus2.plugins.Plugin;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.ui.UIInstance;
import org.gudy.azureus2.plugins.ui.UIManagerListener;
import org.gudy.azureus2.plugins.ui.tables.TableCell;
import org.gudy.azureus2.plugins.ui.tables.TableCellRefreshListener;
import org.gudy.azureus2.plugins.ui.tables.TableColumn;
import org.gudy.azureus2.plugins.ui.tables.TableManager;
import org.gudy.azureus2.plugins.utils.Formatters;
import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;


/** Simple Column Example
 *
 * @author TuxPaper
 */
public class ExtraColumns implements Plugin {
  static final String COLUMNID_TORRENTNAME = "KazgorColumn1";
  static final String COLUMNID_UPETA = "KazgorColumn2";
  static final String COLUMNID_REMAINING = "KazgorColumn3";
  
 
 
  
  Formatters formatter;

  
  PluginInterface plugin_Interface;
  
  
   /** This method is called when the plugin is loaded / initialized
   *
   * @param pluginInterface access to Azureus' plugin interface
   */
    public void initialize(PluginInterface pluginInterface) {

        //set the global plugin_Interface to the local pluginInterface so later classes can use it
        plugin_Interface = pluginInterface;
        
        formatter = pluginInterface.getUtilities().getFormatters();
        TableManager tableManager = pluginInterface.getUIManager().getTableManager();
        TableColumn kazgorColumn;
    
        
        /* Torrent Filename - Add to the Complete(Seeding) column */    
        TorrentNameRefreshListener torrentNameRefreshListener = new TorrentNameRefreshListener();   
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_COMPLETE,COLUMNID_TORRENTNAME);
        kazgorColumn.initialize(TableColumn.ALIGN_LEAD, TableColumn.POSITION_LAST, 150);  
        kazgorColumn.addCellRefreshListener(torrentNameRefreshListener);
        tableManager.addColumn(kazgorColumn);
    
    
        /* Torrent Filename - Add to the Incomplete(downloading) column */
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_INCOMPLETE, COLUMNID_TORRENTNAME);
        kazgorColumn.initialize(TableColumn.ALIGN_LEAD, TableColumn.POSITION_LAST, 150);
        kazgorColumn.addCellRefreshListener(torrentNameRefreshListener);
        tableManager.addColumn(kazgorColumn);
        
        
        /* Eta for Seeding column to get you to a 1:1 ratio */
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_COMPLETE, COLUMNID_UPETA);
        kazgorColumn.initialize(TableColumn.ALIGN_TRAIL,TableColumn.POSITION_LAST, 150, TableColumn.INTERVAL_LIVE);
        kazgorColumn.addCellRefreshListener(new UploadEtaListener());
        tableManager.addColumn(kazgorColumn);
    
        
        /* Eta for Seeding column to get you to a 1:1 ratio */
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_COMPLETE, COLUMNID_REMAINING);
        kazgorColumn.initialize(TableColumn.ALIGN_TRAIL,TableColumn.POSITION_LAST, 150, TableColumn.INTERVAL_LIVE);
        kazgorColumn.addCellRefreshListener(new UploadRemainingListener());
        tableManager.addColumn(kazgorColumn);  

        
        //The magic to open a plugin GUI view -- see, wasn't that easy!
        //plugin_Interface.getUIManager().getSWTManager().addView((new View(pluginInterface)),false);
        
        //update opening the GUI to the new interface
        pluginInterface.getUIManager().addUIListener(
                new UIManagerListener()
                {
                  public void
                  UIAttached(
                          UIInstance instance )
                  {
                      if ( instance instanceof UISWTInstance ){
                          UISWTInstance swt = (UISWTInstance)instance;
                          View view = new View(plugin_Interface);
                          swt.addView(view,false);
                      }
                  }

                 public void UIDetached(UIInstance arg0) {
                     
                     
                 }
                }); 
   }
  

/** Start of the Listener processes */  
    
   public class TorrentNameRefreshListener implements TableCellRefreshListener {
       public void refresh(TableCell cell) {
          Download dm = (Download)cell.getDataSource();
          if (dm == null)
            return;
                  
          String TorrentName = dm.getTorrentFileName();
     
    // Now lets just get the filename minus the path
          File fl = new File (TorrentName);
          TorrentName = fl.getName();
          
          String str = TorrentName;     
          int p = str.lastIndexOf(".torrent");
          
          //if the filename does not end in .torrent it must
          //be autoloaded from azureus itself (like my AZCVSUpdater does)
          //.. therefore it will end in .tmp instead
          if(p == -1){
              p = str.lastIndexOf(".tmp");
    //        if all else fails, just have the full name so you do not get errors
              if(p == -1){
                  p=str.length();
              }
              
          }
          
                
          TorrentName = str.substring(0, p);
      
          if (!cell.setSortValue(TorrentName) && cell.isValid())
            return;
               
          cell.setText(TorrentName);
        }
      }

    /*************************************
    // * Get the ETA for Seeding Uploads *
    // ***********************************/
 
   public class UploadEtaListener implements TableCellRefreshListener {
    
    public void refresh(TableCell cell) {
      Download dm = (Download)cell.getDataSource();
      if (dm==null) return;
            

// For all none seeding torrent dont display anything in the coloum     
      int state = dm.getState(); 
	  if (state != Download.ST_SEEDING)
      {
        if (!cell.setText("") && cell.isValid()) 
        {
			// no cell text to update            
        }
        else
        {
            cell.setSortValue(0);
        }
        return;
      }
      
      long filesize = dm.getTorrent().getSize();
      long uploaded = dm.getStats().getUploaded();     
      float ratio = getRatio();
      if (ratio < 0.2)
          ratio = Float.valueOf("0.2").floatValue();
	  
	  float filesized_ratio = filesize * ratio;
      
      if (uploaded >= filesized_ratio)
      {
            if (!cell.setText("Uploaded >= " + ratio + ":1") && cell.isValid())
            {
				// no cell text to update
            }
            else
            {
                cell.setSortValue(0);   
            }
            return;
      }
 
      long upspeed  = dm.getStats().getUploadAverage();
            
      if ((upspeed <= 0))
      {
            if (!cell.setText("No Eta") && cell.isValid())
            { 
				// no cell text to update
            }
            else
            {
                cell.setSortValue(0);
            }
            return;       
      } 
                            
      float eta = (filesized_ratio - uploaded) / upspeed;
      
      //Really Ugly conversion ;) -- got to get from float to long.. so round it first / then to string / then to long
      eta = Math.round(eta);
      String eta_string = Float.toString(eta).substring(0,Float.toString(eta).indexOf("."));
      long eta_long = Long.parseLong(eta_string);     
      
      if (!cell.setSortValue(eta) && cell.isValid())
      return;      
//-------------
      float remaining = 0;
      if (filesized_ratio >= uploaded)
      {      
         remaining = filesized_ratio - uploaded;
      }
      
      float perc = ((remaining * 100 ) / filesized_ratio);
	  try{
		  if (perc >= 75)
          {
              ((TableCell)cell).setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_RED", 200),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_GREEN",0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_BLUE",0)); 
          }
          else if ((perc >= 50) && (perc < 75)) 
          {
              ((TableCell)cell).setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_RED", 255),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_GREEN",128),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_BLUE",64));
          } 
          else if ((perc >= 25) && (perc < 50)) 
          {
              ((TableCell)cell).setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_RED", 0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_GREEN",128),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_BLUE",255));
          }
          else
          {
                  if(perc != 0.0 && cell.isValid())
                      ((TableCell)cell).setForeground(
                              plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_RED", 64),
                              plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_GREEN",0),
                              plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_BLUE",128));
           }
 	  
	  	}catch (Exception e){
		  	e.printStackTrace();
			}	  
//--------------
      cell.setText(format(eta_long));
	}
  }
   
   /**
    * Formats time like the core does
    * @param time
    * @return
    */
   public static String format(long time)
   {
         
     int secs = (int) time % 60;
     int mins = (int) (time / 60) % 60;
     int hours = (int) (time /3600) % 24;
     int days = (int) (time / 86400);
     
     String result = "";
     if (days > 0) result = days + "d ";
     if (hours > 0 || days > 0) result += hours + "h ";
     if ((days == 0) && ((mins > 0 || hours > 0) && mins < 10)) result += "0" + mins + "m ";
     if ((days == 0) && ((mins > 0 || hours > 0) && mins >= 10)) result += mins + "m ";
     if ((hours == 0 && days == 0) && ((secs > 0 || mins > 0) && secs < 10)) result += "0" + secs + "s ";
     if ((hours == 0 && days == 0) && ((secs > 0 || mins > 0) && secs >= 10)) result += secs + "s ";
     
     return result.length() == 0 ? "" : result.substring(0, result.length() - 1);
   } 

   
   /**
    * Sets the listener for UploadRemaining
    * @author kazgor
    *
    */
   public class UploadRemainingListener implements TableCellRefreshListener {
    public void refresh(TableCell cell) {
      Download dm = (Download)cell.getDataSource();
      if (dm==null) return;

	  float real_download = dm.getStats().getDownloaded();
      float filesize = dm.getTorrent().getSize();
      float uploaded = dm.getStats().getUploaded();
	  
	  float ratio = getRatio();
      if (ratio < 0.2)
        ratio = Float.valueOf("0.2").floatValue();
      
      float filesized_ratio = filesize * ratio;
      
      float remaining = 0;
      if (filesized_ratio >= uploaded)
      {      
         remaining = filesized_ratio - uploaded;
      }
      
      float perc = ((remaining * 100 ) / filesized_ratio);
      
      try{
          if (perc >= 75)
          {
			  ((TableCell)cell).setForeground(
					  plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_RED", 200),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_GREEN",0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_BLUE",0)); 
           }
           else if ((perc >= 50) && (perc < 75)) 
           {
              ((TableCell)cell).setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_RED", 255),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_GREEN",128),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_BLUE",64));
          } 
          else if ((perc >= 25) && (perc < 50)) 
          {
              ((TableCell)cell).setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_RED", 0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_GREEN",128),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_BLUE",255));
          }
          else
          {
			  if(perc != 0.0 && cell.isValid())
				  ((TableCell)cell).setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_RED", 64),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_GREEN",0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_BLUE",128));
          }
         
          int percI = (int) (perc * 100);      
          String PercS = "  (" + (percI / 100) + "." + (percI % 100) +" % )";
          if (!cell.setSortValue((long) remaining) && cell.isValid())
			  return;
                  
          //cell.setText(DisplayFormatters.formatByteCountToKiBEtc((long) remaining)+ PercS);
          cell.setText(formatter.formatByteCountToKiBEtc((long) remaining)+ PercS);

		  String hovertext = "Upload target ("+formatter.formatByteCountToKiBEtc((long) filesized_ratio)+")"; 
		  hovertext = hovertext + "\nUpload Ratio of ("+ratio+":1)";
				  
		  cell.setToolTip(hovertext);
				  
	  }catch (Exception e){
          //If a null point exception is still thrown (had it happen once!?!),
          //then ignore it
          e.printStackTrace();
      }
    }
   }


   /**
    * Method to get the ratio from the user settings
    * @return float
    */
   public float getRatio(){
       try
	   {
           int type = plugin_Interface.getPluginconfig().getPluginIntParameter("ratio_type",2);
           
           if(type == 1)
		   {
			   long setting = plugin_Interface.getPluginconfig().getIntParameter("StartStopManager_iFirstPriority_ShareRatio",1000);
               float float_temp =  (float)setting / 1000;
               return float_temp;
           }
		   else if(type == 2)
		   {               
               float float_temp =plugin_Interface.getPluginconfig().getFloatParameter("Stop Ratio"); 
               return float_temp;
           }
		   else
		   {
			   return getManualRatio();
           }          
           
       } 
	   catch (Exception e)
	   {
           e.printStackTrace();
		   float float_temp = (float) 1.0;
           return float_temp;
       }
       
   }
 
   /**
    * Method to get the Manual set Ration out of a String stored value 
    * @return float ratio
    */
   public float getManualRatio(){
       try{
           Float ratio =  Float.valueOf(plugin_Interface.getPluginconfig().getPluginStringParameter("manual_ratio","2"));
           return ratio.floatValue();
       }catch(Exception e){
           e.printStackTrace();
           return 2;
       }
       
   }

}//EOF
